# coding: utf-8
import uuid

from .department import Department
from .department_crud import DepartmentCRUD
from .error_model import Error
from . import Singleton

import logging

class DuplicateError(Error):
    """
    Error - Duplicate Department
    """ 
    def __init__(self, key=""):
        Error.__init__(self,
                       status=400,
                       title="Duplicate Department",
                       type="RG-001",
                       key=key)

    def __str__(self):
        return "%s already exist" % self._key

class NotDepartmentError(Error):
    """
    Error - Department Error
    """
    def __init__(self):
        Error.__init__(self,
                       status=405,
                       title="Department error",
                       type="INPUT-003")

    def __str__(self):
        return "It is not Department or DepartmentCRUD object"

class NotFoundDepartmentError(Error):
    """
    Error - Not Found Department
    """
    def __init__(self, key=""):
        Error.__init__(self,
                       status=404,
                       title="Not Found Department",
                       type="RG-002",
                       key=key)

    def __str__(self):
        return "Department %s not found" % self._key

class DepartmentNotEmptyError(Error):
    """
    Error - Not Empty Department
    """
    def __init__(self, key=""):
        Error.__init__(self,
                       status=404,
                       title="Not Empty Department",
                       type="RG-003",
                       key=key)

    def __str__(self):
        return "Department %s not empty" % self._key


class Departments(list, metaclass=Singleton):
    """
    Departments - manage list of Department
    """
    def __init__(self):
        list.__init__(self)
    
    def _get_id(self):
        return uuid.uuid1().int  

    def append(self, el):
        if type(el) is DepartmentCRUD:
            el = Department.from_dict(dict(id=self._get_id(), **el.to_dict())) 
        if type(el) is Department:
            if el in self:
                raise DuplicateError(el)
            list.append(self,el)
            logging.getLogger('connexion.app').warning("create department %s" % el.id)
            return el
        raise NotDepartmentError()
    
    def getDepartmentById(self,id):
        for department in self:
            if department.id == id:
                return department
        raise NotFoundDepartmentError(id)        

    def update(self, el):
        if type(el) is not Department:
            raise NotDepartmentError()
        self.remove(self.getDepartmentById(el.id))
        self.append(el)
        return el
    
    def remove(self, el):
        if el.cntpeople > 0:
            raise DepartmentNotEmptyError()
        list.remove(self,el)
        logging.getLogger('connexion.app').warning("remove department %s" % el.id)
